/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.l2j.gameserver.clientpackets;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.logging.Logger;

import net.sf.l2j.Config;
import net.sf.l2j.L2DatabaseFactory;
import net.sf.l2j.gameserver.Olympiad;
import net.sf.l2j.gameserver.SevenSignsFestival;
import net.sf.l2j.gameserver.communitybbs.Manager.RegionBBSManager;
import net.sf.l2j.gameserver.datatables.SkillTable;
import net.sf.l2j.gameserver.instancemanager.CustomZoneManager;
import net.sf.l2j.gameserver.model.L2Party;
import net.sf.l2j.gameserver.model.L2World;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.model.entity.TvTEvent;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.FriendList;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.taskmanager.AttackStanceTaskManager;

import net.sf.l2j.gameserver.network.L2GameClient;

/**
 * This class ...
 *
 * @version $Revision: 1.9.4.3 $ $Date: 2005/03/27 15:29:30 $
 */
public final class Logout extends L2GameClientPacket
{
	private static final String _C__09_LOGOUT = "[C] 09 Logout";
	private static Logger _log = Logger.getLogger(Logout.class.getName());


	@Override
	protected void readImpl()
	{

	}

	@Override
	protected void runImpl()
	{
		// Dont allow leaving if player is fighting
		L2PcInstance player = getClient().getActiveChar();

		if (player == null)
			return;

        // [L2J_JP ADD START]
		if (!(player.isGM()))
        {
            if(CustomZoneManager.getInstance().checkIfInZone("NoEscape", player)){
                player.sendPacket(SystemMessage.sendString("You can not log out in here."));
                player.sendPacket(ActionFailed.STATIC_PACKET);
                return;                   
            }
        }
	
        if(player.isFlying())
        {
            player.sendPacket(SystemMessage.sendString("You can not log out while flying."));
            player.sendPacket(ActionFailed.STATIC_PACKET);
            return;                   
        }
        // [L2J_JP ADD END]

		player.getInventory().updateDatabase();

		if(AttackStanceTaskManager.getInstance().getAttackStanceTask(player))
		{
			if (Config.DEBUG) _log.fine("Player " + player.getName() + " tried to logout while fighting");

			player.sendPacket(new SystemMessage(SystemMessageId.CANT_LOGOUT_WHILE_FIGHTING));
			player.sendPacket(ActionFailed.STATIC_PACKET);
			return;
		}

		if(player.atEvent)
		{
			player.sendMessage("A superior power doesn't allow you to leave the event");
			return;
		}

        if (player.isInOlympiadMode() || Olympiad.getInstance().isRegistered(player))
        {
            player.sendMessage("You cant logout in olympiad mode");
            return;
        }

		// Prevent player from logging out if they are a festival participant
		// and it is in progress, otherwise notify party members that the player
		// is not longer a participant.
		if (player.isFestivalParticipant()) {
			if (SevenSignsFestival.getInstance().isFestivalInitialized())
			{
				player.sendMessage("You cannot log out while you are a participant in a festival.");
				return;
			}
			L2Party playerParty = player.getParty();

			if (playerParty != null)
				player.getParty().broadcastToPartyMembers(SystemMessage.sendString(player.getName() + " has been removed from the upcoming festival."));
		}
		if (player.isFlying())
		{
			player.removeSkill(SkillTable.getInstance().getInfo(4289, 1));
		}

		TvTEvent.onLogout(player);
		RegionBBSManager.getInstance().changeCommunityBoard();

		//TODO:[JOJO] ]r
		if (player.isInStoreMode()) {	// lXo܂܃OAEgH
			if (false) {
			//	System.out.println("***"+CallTree.getClassName(this)+".runImp(): "+player.getName()+" ̓]r [hɓ܂B");
				String msg;
				switch(player.getPrivateStoreType()) {
				case L2PcInstance.STORE_PRIVATE_SELL:
						msg = ((msg = player.getSellList().getTitle()) == null) ? "yuz" : msg + "\nyuz";
						player.getSellList().setTitle(msg);
						break;
				case L2PcInstance.STORE_PRIVATE_BUY:
						msg = ((msg = player.getBuyList().getTitle()) == null) ? "yuz" : msg + "\nyuz";
						player.getBuyList().setTitle(msg);
						break;
				case L2PcInstance.STORE_PRIVATE_MANUFACTURE:
						msg = ((msg = player.getCreateList().getStoreName()) == null) ? "yuz" : msg + "\nyuz";
						player.getCreateList().setStoreName(msg);
						break;
				}
			}
			/* vC[IuWFNg̖͂ɁAQ[NCAg̐ؒf݂̂Ȃ */
			L2GameClient.saveCharToDisk(player);
			getClient().isZombie = true;	// vC[IuWFNg𖕏Ȃ߂̃tO
			player.closeNetConnection();	// ؒf ---> vC[IuWFNgĂ΂邪AisZombie tOŖĂB
			notifyFriends(player);
			return;
		}
		//TODO:[/JOJO]

		player.deleteMe();
		notifyFriends(player);
	}

	private void notifyFriends(L2PcInstance cha)
	{
		java.sql.Connection con = null;

		try {
			con = L2DatabaseFactory.getInstance().getConnection();
			PreparedStatement statement;
			statement = con.prepareStatement("SELECT friend_name FROM character_friends WHERE charId=?");
			statement.setInt(1, cha.getObjectId());
			ResultSet rset = statement.executeQuery();

			L2PcInstance friend;
			String friendName;

			while (rset.next())
			{
				friendName = rset.getString("friend_name");

				friend = L2World.getInstance().getPlayer(friendName);

				if (friend != null) //friend logged in.
				{
					friend.sendPacket(new FriendList(friend));
				}
			}

			rset.close();
			statement.close();
		}
		catch (Exception e) {
			_log.warning("could not restore friend data:"+e);
		}
		finally {
			try {con.close();} catch (Exception e){}
		}
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.clientpackets.ClientBasePacket#getType()
	 */
	@Override
	public String getType()
	{
		return _C__09_LOGOUT;
	}
}