/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.l2j.gameserver.handler.chathandlers;

import java.util.StringTokenizer;
import java.util.logging.Logger;

import net.sf.l2j.Config;
import net.sf.l2j.gameserver.handler.IChatHandler;
import net.sf.l2j.gameserver.handler.IVoicedCommandHandler;
import net.sf.l2j.gameserver.handler.VoicedCommandHandler;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.serverpackets.CreatureSay;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;

/**
 * A chat handler
 *
 * @author  durgus
 */
public class ChatAll implements IChatHandler
{
	private static final int[] COMMAND_IDS = { 0 };
	private static Logger _log = Logger.getLogger(ChatAll.class.getName());

	/**
	 * Handle chat type 'all'
	 * @see net.sf.l2j.gameserver.handler.IChatHandler#handleChat(int, net.sf.l2j.gameserver.model.actor.instance.L2PcInstance, java.lang.String)
	 */
	public void handleChat(int type, L2PcInstance activeChar, String target, String text)
	{
		if (text.startsWith("."))
		{
			//TODO:[JOJO]
			if (text.startsWith(".title=")) {
				//|if (activeChar.reduceAdena("setTitle", /*z=>*/1000, null, true)) {
				//|if (activeChar.destroyItem("setTitle", /*ACe=>*/1458, /*=>*/10, null, true)) {
				if (activeChar.getLevel() >= 20) {
					activeChar.setTitle(text.substring(7).trim());
					activeChar.sendPacket(new SystemMessage(SystemMessageId.TITLE_CHANGED));
					activeChar.broadcastUserInfo();
				}
				return;
			}
			if (text.equals(".server_restart")) {
				//- new net.sf.l2j.custom.VoteServerRestart().popup(activeChar);
				net.sf.l2j.gameserver.communitybbs.Manager.FavoriteManager.getInstance().getRestartServerHTML(activeChar);
				return;
			}
			//TODO:[JOJO]

			StringTokenizer st = new StringTokenizer(text);
			IVoicedCommandHandler vch;
			String command = "";

			if (st.countTokens() > 1)
			{
				command = st.nextToken().substring(1);
				target = text.substring(command.length() + 2);
				vch = VoicedCommandHandler.getInstance().getVoicedCommandHandler(command);
			}
			else
			{
				command = text.substring(1);
				if (Config.DEBUG) _log.info("Command: "+command);
				vch = VoicedCommandHandler.getInstance().getVoicedCommandHandler(command);
			}
			if (vch != null)
			{
				vch.useVoicedCommand(command, activeChar, target);
			}
			else
			{
				if (Config.DEBUG) _log.warning("No handler registered for bypass '"+command+"'");
			}
		}
		else
		{
			CreatureSay cs = new CreatureSay(activeChar.getObjectId(), type, activeChar.getAppearance().getVisibleName(), text);

			for (L2PcInstance player : activeChar.getKnownList().getKnownPlayers().values())
			{
				if (player != null && activeChar.isInsideRadius(player, 1250, false, true))
				{
					player.sendPacket(cs);
				}
			}

			activeChar.sendPacket(cs);
		}
	}

	/**
	 * Returns the chat types registered to this handler
	 * @see net.sf.l2j.gameserver.handler.IChatHandler#getChatTypeList()
	 */
	public int[] getChatTypeList()
	{
		return COMMAND_IDS;
	}
}